@@ -684,6 +684,12 @@ andAttribute: (EOAttribute *)_attribute
   if (!sqlType)
     sqlType = [self _sqlTypeForColumn: _field
                     withFieldInfos: [folderInfo fields]];
+  if (!sqlType && [_field isEqualToString: @"c_folder_id"])
+    // The c_folder_id column used in single-store mode is not defined
+    // in the OCS files; return the definition from the c_version column
+    // which uses the same data type.
+    sqlType = [self _sqlTypeForColumn: @"c_version"
+                       withFieldInfos: [folderInfo fields]];
   if (sqlType)
     {
       attribute = AUTORELEASE([[EOAttribute alloc] init]);
@@ -718,8 +724,6 @@ andAttribute: (EOAttribute *)_attribute
   [sql appendString:@"INSERT INTO "];
   [sql appendString:_table];
   [sql appendString:@" ("];
-  if ([GCSFolderManager singleStoreMode])
-    [sql appendString:@"c_folder_id, "];
 
   for (i = 0, count = [keys count]; i < count; i++) {
     if (i != 0) [sql appendString:@", "];
@@ -727,8 +731,6 @@ andAttribute: (EOAttribute *)_attribute
   }
 
   [sql appendString:@") VALUES ("];
-  if ([GCSFolderManager singleStoreMode])
-    [sql appendFormat:@"%@, ", folderId];
 
   for (i = 0, count = [keys count]; i < count; i++) {
     fieldName = [keys objectAtIndex:i];
@@ -1094,6 +1096,12 @@ andAttribute: (EOAttribute *)_attribute
 
 		      if (isNewRecord)
 			{
+                          if ([GCSFolderManager singleStoreMode])
+                            {
+                              [contentRow setObject: folderId  forKey:@"c_folder_id"];
+                              [quickRow setObject: folderId  forKey:@"c_folder_id"];
+                            }
+
 			  if (!ofFlags.sameTableForQuick)
 			    error = (hasInsertDelegate
 				     ? [quickChannel insertRowX: quickRow forEntity: quickTableEntity]
